@@ -65,6 +65,14 @@ class ParsedUrl:
 		self.set_server_path()
 		self.set_host_user_port()
 
+	def straight_url(self):
+		"""Return the URL with the username stripped"""
+		# Emulation of the ternary operator:
+		strpath = (self.path != None and self.path or "")
+		strport = (self.port != None and (":%d"%self.port) or "")
+		url = '%s://%s%s/%s' % (self.protocol, self.host, strport, strpath)
+		return url
+
 	def bad_url(self, message = None):
 		"""Report a bad url, using message if given"""
 		if message:
@@ -104,8 +112,8 @@ class ParsedUrl:
 		# Set user and host
 		user_comps = user_host.split("@")
 		if len(user_comps) >= 2:
-			self.user = user_comps[0]
-			self.host = "@".join(user_comps[1:])
+			self.user = "@".join(user_comps[0:-1])
+			self.host = user_comps[-1]
 		else: self.host = user_host
 
 
@@ -318,8 +326,8 @@ sftp_command = "sftp"
 # default to batch mode using public-key encryption
 ssh_askpass = False
 
-# this keeps us from having to mess with the prompts regarding known hosts
-ssh_options = "-oUserKnownHostsFile=/dev/null -oStrictHostKeyChecking=no"
+# user added ssh options
+ssh_options = ""
 
 class sshBackend(Backend):
 	"""This backend copies files using scp.  List not supported"""
@@ -330,9 +338,14 @@ class sshBackend(Backend):
 			import pexpect
 			self.pexpect = pexpect
 		except ImportError:
-			raise BackendException("This backend requires the pexpect module, "
-								   "pexpect from http://pexpect.sourceforge.net or "
-								   "python-pexpect from your distro's repository.")
+			self.pexpect = None
+		if not (self.pexpect and
+				hasattr(self.pexpect, '__version__') and
+				self.pexpect.__version__ >= '2.1'):
+			log.FatalError("This backend requires the pexpect module version 2.1 or later."
+						   "You can get pexpect from http://pexpect.sourceforge.net or "
+						   "python-pexpect from your distro's repository.")
+		
 		# host string of form user@hostname:port
 		self.host_string = parsed_url.server.split(':')[0]
 		# make sure remote_dir is always valid
@@ -366,9 +379,11 @@ class sshBackend(Backend):
 				if state == "authorizing":
 					match = child.expect([self.pexpect.EOF,
 										  self.pexpect.TIMEOUT,
-										  "(?i)password",
-										  "(?i)permission denied"],
+										  "(?i)password:",
+										  "(?i)permission denied",
+										  "authenticity"],
 										 timeout = globals.timeout)
+					log.Log("State = %s, Before = '%s'" % (state, child.before.strip()), 9)
 					if match == 0:
 						log.Log("Failed to authenticate", 5)
 						break
@@ -381,12 +396,17 @@ class sshBackend(Backend):
 					elif match == 3:
 						log.Log("Invalid SSH password", 1)
 						break
+					elif match == 4:
+						log.Log("Remote host authentication failed (missing known_hosts entry?)", 1)
+						break
 				elif state == "copying":
 					match = child.expect([self.pexpect.EOF,
 										  self.pexpect.TIMEOUT,
 										  "stalled",
+										  "authenticity",
 										  "ETA"],
 										 timeout = globals.timeout)
+					log.Log("State = %s, Before = '%s'" % (state, child.before.strip()), 9)
 					if match == 0:
 						break
 					elif match == 1:
@@ -394,11 +414,15 @@ class sshBackend(Backend):
 						break
 					elif match == 2:
 						state = "stalled"
+					elif match == 3:
+						log.Log("Remote host authentication failed (missing known_hosts entry?)", 1)
+						break
 				elif state == "stalled":
 					match = child.expect([self.pexpect.EOF,
 										  self.pexpect.TIMEOUT,
 										  "ETA"],
 										 timeout = globals.timeout)
+					log.Log("State = %s, Before = '%s'" % (state, child.before.strip()), 9)
 					if match == 0:
 						break
 					elif match == 1:
@@ -424,9 +448,11 @@ class sshBackend(Backend):
 				match = child.expect([self.pexpect.EOF,
 									  self.pexpect.TIMEOUT,
 									  "sftp>",
-									  "(?i)password",
+									  "(?i)password:",
 									  "(?i)permission denied",
+									  "authenticity",
 									  "(?i)no such file or directory"])
+				log.Log("State = sftp, Before = '%s'" % (child.before.strip()), 9)
 				if match == 0:
 					break
 				elif match == 1:
@@ -447,6 +473,9 @@ class sshBackend(Backend):
 					log.Log("Invalid SSH password", 1)
 					break
 				elif match == 5:
+					log.Log("Host key authenticity could not be verified (missing known_hosts entry?)", 1)
+					break
+				elif match == 6:
 					log.Log("Remote file or directory '%s' does not exist" % self.remote_dir, 1)
 					break
 			child.close()
@@ -502,30 +531,59 @@ class ftpBackend(Backend):
 	"""Connect to remote store using File Transfer Protocol"""
 	def __init__(self, parsed_url):
 		Backend.__init__(self, parsed_url)
-		self.url_string = parsed_url.url_string
+
+		# we expect an error return, so go low-level and ignore it
+		try:
+			p = os.popen("ncftpls -v")
+			fout = p.read()
+			ret = p.close()
+		except:
+			pass
+		# the expected error is 8 in the high-byte and some output
+		if ret != 0x0800 or not fout:
+			log.FatalError("NcFTP not found:  Please install NcFTP version 3.1.9 or later")
+
+		# version is the second word of the first line
+		version = fout.split('\n')[0].split()[1]
+		if version < "3.1.9":
+			log.FatalError("NcFTP too old:  Duplicity requires NcFTP version 3.1.9 or later")
+		log.Log("NcFTP version is %s" % version, 4)
+
+		self.url_string = parsed_url.straight_url()
 		if self.url_string[-1] != '/':
 			self.url_string += '/'
+
 		self.password = self.get_password()
+
 		if globals.ftp_connection == 'regular':
 			self.conn_opt = '-E'
 		else:
 			self.conn_opt = '-F'
-		self.flags = "%s -t %s -u '%s' -p '%s'" % \
-					 (self.conn_opt, globals.timeout, parsed_url.user, self.password)
+
+ 		self.tempfile, self.tempname = tempfile.mkstemp("", "duplicity.")
+ 		atexit.register(os.unlink, self.tempname)
+		os.write(self.tempfile, "host %s\n" % parsed_url.host)
+ 		os.write(self.tempfile, "user %s\n" % parsed_url.user)
+ 		os.write(self.tempfile, "pass %s\n" % self.password)
+ 		os.close(self.tempfile)
+		self.flags = "-f %s %s -t %s" % \
+					 (self.tempname, self.conn_opt, globals.timeout)
+		if parsed_url.port != None and parsed_url.port != 21:
+			self.flags += " -P '%s'" % (parsed_url.port)
 
 	def put(self, source_path, remote_filename = None):
 		"""Transfer source_path to remote_filename"""
 		pu = ParsedUrl(self.url_string)
 		remote_path = os.path.join (pu.path, remote_filename).rstrip()
-		commandline = "ncftpput %s -V -c '%s' '%s' < '%s'" % \
-					  (self.flags, pu.host, remote_path, source_path.name)
+		commandline = "ncftpput %s -V -C '%s'  '%s'" % \
+					  (self.flags, source_path.name, remote_path)
 		self.run_command_persist(commandline)
 
 	def get(self, remote_filename, local_path):
 		"""Get remote filename, saving it to local_path"""
 		pu = ParsedUrl(self.url_string)
 		remote_path = os.path.join(pu.path, remote_filename).rstrip()
-		commandline = "ncftpget %s -V -c '%s' '%s' > '%s'" % \
+		commandline = "ncftpget %s -V -C '%s' '%s' '%s'" % \
 					  (self.flags, pu.host, remote_path, local_path.name)
 		self.run_command_persist(commandline)
 		local_path.setdata()
@@ -534,8 +592,8 @@ class ftpBackend(Backend):
 		"""List files in directory"""
 		pu = ParsedUrl(self.url_string)
 		# we create the directory first so we have target dir
-		commandline = "ncftpls %s -W 'MKD %s' 'ftp://%s'" % \
-					  (self.flags, pu.path, pu.host)
+		commandline = "ncftpls %s -W 'MKD %s' '%s'" % \
+					  (self.flags, pu.path, self.url_string)
 		l = self.popen_persist(commandline).split('\n')
 		# try for a long listing to avoid connection reset
 		commandline = "ncftpls %s -l '%s'" % \
@@ -545,7 +603,7 @@ class ftpBackend(Backend):
 		if not l:
 			return l
 		# if long list is not empty, get short list of names only
-		commandline = "ncftpls %s '%s'" % \
+		commandline = "ncftpls %s -1 '%s'" % \
 					  (self.flags, self.url_string)
 		l = self.popen_persist(commandline).split('\n')
 		return filter(lambda x: x, l)
@@ -556,7 +614,7 @@ class ftpBackend(Backend):
 		for filename in filename_list:
 			commandline = "ncftpls %s -X 'DELE %s' '%s'" % \
 						  (self.flags, filename, self.url_string)
-			self.run_command_persist(commandline)
+			self.popen_persist(commandline)
 
 
 class rsyncBackend(Backend):
@@ -668,21 +726,39 @@ class BotoBackend(Backend):
 	def put(self, source_path, remote_filename=None):
 		if not remote_filename:
 			remote_filename = source_path.get_filename()
-		log.Log("Uploading %s to Amazon S3" % remote_filename, 5)
 		key = self.key_class(self.bucket)
 		key.key = remote_filename
-		key.set_contents_from_filename(source_path.name, 
-				{'Content-Type': 'application/octet-stream'})
+		for n in range(1, globals.num_retries+1):
+			log.Log("Uploading %s to Amazon S3 (attempt #%d)" % (remote_filename, n), 5)
+			try:
+				key.set_contents_from_filename(source_path.name, 
+					{'Content-Type': 'application/octet-stream'})
+				return
+			except:
+				pass
+			log.Log("Uploading %s failed (attempt #%d)" % (remote_filename, n), 1)
+			time.sleep(30)
+		log.Log("Giving up trying to upload %s after %d attempts" % (remote_filename, globals.num_retries), 1)
+		raise BackendException("Error uploading %s" % remote_filename)
 	
 	def get(self, remote_filename, local_path):
-		log.Log("Downloading %s from Amazon S3" % remote_filename, 5)
 		key = self.key_class(self.bucket)
 		key.key = remote_filename
-		key.get_contents_to_filename(local_path.name)
-		local_path.setdata()
+		for n in range(1, globals.num_retries+1):
+			log.Log("Downloading %s from Amazon S3 (attempt #%d)" % (remote_filename, n), 5)
+			try:
+				key.get_contents_to_filename(local_path.name)
+				local_path.setdata()
+				return
+			except:
+				pass
+			log.Log("Downloading %s failed (attempt #%d)" % (remote_filename, n), 1)
+			time.sleep(30)
+		log.Log("Giving up trying to download %s after %d attempts" % (remote_filename, globals.num_retries), 1)
+		raise BackendException("Error downloading %s" % remote_filename)
 
 	def list(self):
-		filename_list = [k.key for k in self.bucket.get_all_keys()]
+		filename_list = [k.key for k in self.bucket]
 		log.Log("Files in bucket:\n%s" % '\n'.join(filename_list), 9)
 		return filename_list
 
@@ -744,18 +820,22 @@ class webdavBackend(Backend):
 		
 	def list(self):
 		"""List files in directory"""
-		log.Log("Listing directory %s on WebDAV server" % (self.directory,), 5)
-		self.headers['Depth'] = "1"
-		self.conn.request("PROPFIND", self.directory, self.listbody, self.headers)
-		del self.headers['Depth']
-		response = self.conn.getresponse()
-		if response.status != 207:
-			raise BackendException((response.status, response.reason))
+		for n in range(1, globals.num_retries+1):
+			log.Log("Listing directory %s on WebDAV server" % (self.directory,), 5)
+			self.headers['Depth'] = "1"
+			self.conn.request("PROPFIND", self.directory, self.listbody, self.headers)
+			del self.headers['Depth']
+			response = self.conn.getresponse()
+			if response.status == 207:
+				document = response.read()
+				break
+			log.Log("WebDAV PROPFIND attempt #%d failed: %s %s" % (n, response.status, response.reason), 5)
+			if n == globals.num_retries +1:
+				log.Log("WebDAV backend giving up after %d attempts to PROPFIND %s" % (globals.num_retries, self.directory), 1)
+				raise BackendException((response.status, response.reason))
 
-		document = response.read()
-		print document
+		log.Log("%s" % (document,), 6)
 		dom = xml.dom.minidom.parseString(document)
-
 		result = []
 		for href in dom.getElementsByTagName('D:href'):
 			filename = urllib.unquote(self._getText(href.childNodes).strip())
@@ -767,15 +847,19 @@ class webdavBackend(Backend):
 	def get(self, remote_filename, local_path):
 		"""Get remote filename, saving it to local_path"""
 		url = self.directory + remote_filename
-		log.Log("Retrieving %s from FTP server" % (url ,), 5)
 		target_file = local_path.open("wb")
-		self.conn.request("GET", url, None, self.headers)
-		response = self.conn.getresponse()		
-		if response.status != 200:
-			raise BackendException((response.status, response.reason))
-		target_file.write(response.read())
-		assert not target_file.close()
-		local_path.setdata()
+		for n in range(1, globals.num_retries+1):
+			log.Log("Retrieving %s from WebDAV server" % (url ,), 5)
+			self.conn.request("GET", url, None, self.headers)
+			response = self.conn.getresponse()		
+			if response.status == 200:
+				target_file.write(response.read())
+				assert not target_file.close()
+				local_path.setdata()
+				return
+			log.Log("WebDAV GET attempt #%d failed: %s %s" % (n, response.status, response.reason), 5)
+		log.Log("WebDAV backend giving up after %d attempts to GET %s" % (globals.num_retries, url), 1)
+		raise BackendException((response.status, response.reason))
 
 	def put(self, source_path, remote_filename = None):
 		"""Transfer source_path to remote_filename"""
@@ -783,25 +867,33 @@ class webdavBackend(Backend):
 			remote_filename = source_path.get_filename()
 		url = self.directory + remote_filename
 		source_file = source_path.open("rb")
-		log.Log("Saving %s on WebDAV server" % (url,), 5)
-		self.conn.request("PUT", url, source_file.read(), self.headers)
-		response = self.conn.getresponse()
-		if response.status != 201:
-			raise BackendException((response.status, response.reason))
-		response.read()
-		assert not source_file.close()
+		for n in range(1, globals.num_retries+1):
+			log.Log("Saving %s on WebDAV server" % (url ,), 5)
+			self.conn.request("PUT", url, source_file.read(), self.headers)
+			response = self.conn.getresponse()
+			if response.status == 201:
+				response.read()
+				assert not source_file.close()
+				return
+			log.Log("WebDAV PUT attempt #%d failed: %s %s" % (n, response.status, response.reason), 5)
+		log.Log("WebDAV backend giving up after %d attempts to PUT %s" % (globals.num_retries, url), 1)
+		raise BackendException((response.status, response.reason))
 
 	def delete(self, filename_list):
 		"""Delete files in filename_list"""
 		for filename in filename_list:
 			url = self.directory + filename
-			log.Log("Deleting %s from WebDAV server" % (url,), 5)
-			self.conn.request("DELETE", url, None, self.headers)
-			response = self.conn.getresponse()
-			if response.status != 204:
-				raise BackendException((response.status, response.reason))
-			response.read()
-
+			for n in range(1, globals.num_retries+1):
+				log.Log("Deleting %s from WebDAV server" % (url ,), 5)
+				self.conn.request("DELETE", url, None, self.headers)
+				response = self.conn.getresponse()
+				if response.status == 204:
+					response.read()
+					break
+				log.Log("WebDAV DELETE attempt #%d failed: %s %s" % (n, response.status, response.reason), 5)
+				if n == globals.num_retries +1:
+					log.Log("WebDAV backend giving up after %d attempts to DELETE %s" % (globals.num_retries, url), 1)
+					raise BackendException((response.status, response.reason))
 
 hsi_command = "hsi"
 class hsiBackend(Backend):
