@@ -231,6 +231,10 @@ class BackupChain:
 		if self.incset_list: return self.incset_list[-1]
 		else: return self.fullset
 
+	def get_first(self):
+		"""Return first BackupSet in chain (ie the full backup)"""
+		return self.fullset
+
 	def __str__(self):
 		"""Return string representation, for testing purposes"""
 		set_schema = "%20s   %30s   %15s"
@@ -378,11 +382,6 @@ class CollectionsStatus:
 		l = ["Connecting with backend: %s" %
 			 (self.backend.__class__.__name__,),
 			 "Archive dir: %s" % (self.archive_dir,)]
-		if self.matched_chain_pair:
-			l.append("\nFound a complete backup chain with matching "
-					 "signature chain:")
-			l.append(str(self.matched_chain_pair[1]))
-		else: l.append("No backup chains with active signatures found")
 
 		l.append("\nFound %d backup chains without signatures."
 				 % len(self.other_backup_chains))
@@ -392,6 +391,12 @@ class CollectionsStatus:
 			l.append(str(self.other_backup_chains[i]))
 			l.append("")
 
+		if self.matched_chain_pair:
+			l.append("\nFound a complete backup chain with matching "
+					 "signature chain:")
+			l.append(str(self.matched_chain_pair[1]))
+		else: l.append("No backup chains with active signatures found")
+
 		if self.orphaned_backup_sets or self.incomplete_backup_sets:
 			l.append("Also found %d backup sets not part of any chain,"
 					 % (len(self.orphaned_backup_sets),))
@@ -406,7 +411,7 @@ class CollectionsStatus:
 	def set_values(self, sig_chain_warning = 1):
 		"""Set values from archive_dir and backend.
 
-		if archive_dir is None, omit any local chains.  Returs self
+		if archive_dir is None, omit any local chains.  Returns self
 		for convenience.  If sig_chain_warning is set to None, do not
 		warn about unnecessary sig chains.  This is because there may
 		naturally be some unecessary ones after a full backup.
@@ -423,10 +428,10 @@ class CollectionsStatus:
 
 		if self.archive_dir:
 			local_sig_chains, local_orphaned_sig_names = \
-							  self.get_signature_chains(local = 1)
+							  self.get_signature_chains(True)
 		else: local_sig_chains, local_orphaned_sig_names = [], []
 		remote_sig_chains, remote_orphaned_sig_names = \
-						   self.get_signature_chains(0, backend_filename_list)
+						   self.get_signature_chains(False, filelist = backend_filename_list)
 		self.orphaned_sig_names = (local_orphaned_sig_names +
 								   remote_orphaned_sig_names)
 		self.set_matched_chain_pair(local_sig_chains + remote_sig_chains,
@@ -482,8 +487,8 @@ class CollectionsStatus:
 		if self.other_sig_chains and sig_chain_warning:
 			if self.matched_chain_pair:
 				log.Log("Warning, found unnecessary signature chain(s)", 2)
-			else: log.FatalError("Found signatures but no corresponding "
-								 "backup files")
+			else: log.Log("Warning, found signatures but no corresponding "
+								 "backup files", 2)
 
 		if self.incomplete_backup_sets:
 			log.Log("Warning, found incomplete backup sets, probably left "
@@ -555,8 +560,8 @@ class CollectionsStatus:
 
 		def get_new_sigchain():
 			"""Return new empty signature chain"""
-			if local: return SignatureChain(1, self.archive_dir)
-			else: return SignatureChain(None, self.backend)
+			if local: return SignatureChain(True, self.archive_dir)
+			else: return SignatureChain(False, self.backend)
 
 		# Build initial chains from full sig filenames
 		chains, new_sig_filenames = [], []
@@ -661,6 +666,18 @@ class CollectionsStatus:
 		"""Return a list of chains older than time t"""
 		assert self.values_set
 		return filter(lambda c: c.end_time < t, self.all_backup_chains)
+	
+	def get_last_full_backup(self):
+		"""Return the time of the last full backup of the collection, or 0"""
+		assert self.values_set
+		# Is there a nicer way to do that with a lambda function?
+		if len(self.all_backup_chains) == 0:
+			return 0
+		last = self.all_backup_chains[0]
+		for bc in self.all_backup_chains:
+			if bc.get_first().time > last.get_first().time:
+				last = bc
+		return last.get_first().time
 
 	def get_older_than(self, t):
 		"""Returns a list of backup sets older than the given time t
